#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.
''' Command sub-module of the setuptools-qmesh package.

Module with extensions to setuptools.setup package. The extensions also
enable command-line extensions to the setup.py scripts. This module
includes commands that:
Check for a qgis installation
Check for a gmsh installation
Check and install non-PyPI Python packages (currently pyrdm)
Egg-info (file) writers that contain the qmesh licence, version number,
author/maintainer details and git sha key.
'''



#!/usr/bin/env python

#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.
'''Module containing tests of setuptools-qmesh command line options.

This module tests that all command line options introduced by setuptools-qmesh
complete, with a zero-code returned.
'''

import unittest

class TestCommandLineExtensions(unittest.TestCase):
    '''Test class for setuptools-qmesh command line extensions.

    Class (derived from unittest.TestCase) testing that all command line
    options introduced by setuptools-qmesh complete, with a zero-code returned.
    '''

    def setUp(self):
        '''Method setting-up test environment.

        This method is *not* intended to be invoked by the user. It stores the
        path of the testing function, the project root path (setuptools-qmesh)
        and the path of the command configuration file.
        '''
        import os
        import sys
        import logging
        #Create logger object
        self.log = logging.log
        #Construct absolute paths to test directory and project root directory
        self.this_path = os.path.dirname(os.path.realpath(__file__))
        self.project_test_path = os.path.split(self.this_path)[0]
        self.project_root_path = os.path.split(self.project_test_path)[0]
        #Detect python version used to run test, as a string in the
        # form 'major.minor'
        python_version_info = sys.version_info
        self.python_version = str(python_version_info.major) + \
                              '.' + str(python_version_info.minor)

    def test_help_commands(self):
        '''Method testing the setup.py script with the --help-commands option.
        '''
        import os
        #Construct path to the setup.py script
        setup_script_path = os.path.join(self.project_root_path, 'setup.py')
        #Assemble string of command to test
        command = ['python' + self.python_version, setup_script_path, '--help-commands']
        #Invoke command on setup.py script
        self.invoke_and_test_command(command)

    def invoke_and_test_command(self, command):
        '''Method invoking setuptools commands and checking return codes.

        The setuptools commands are issued using the Python version that
        runs the test.
        '''
        import tempfile
        import subprocess
        #Invoke command and capture stdout & stderr to temporary files
        command_stdout = tempfile.TemporaryFile(mode='w+b')
        command_stderr = tempfile.TemporaryFile(mode='w+b')
        command_proc = subprocess.Popen(command,
                                        stdout=command_stdout,
                                        stderr=command_stderr)
        command_proc.wait()
        #If command had issues, echo its stdout and stdout
        if command_proc.returncode != 0:
            command_stdout.seek(0)
            self.log(40, '***command stdout:')
            for line in command_stdout:
                self.log(40, line.decode('utf-8').strip('\n'))
            command_stderr.seek(0)
            self.log(40, '***command stderr:')
            for line in command_stderr:
                self.log(40, line.decode('utf-8').strip('\n'))
        #Close files
        command_stdout.close()
        command_stderr.close()
        #Make sure command has run without issues
        self.assertEqual(command_proc.returncode, 0,
                         'The command '+ ' '.join(command) +\
                         ' returned a non-zero code: '+str(command_proc.returncode)+'.'+\
                         ' Please check command output.')



#
#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.

'''Functions writing egg-info files.

The map ``entry_points`` in ``setuptools.setup`` contains a ``egg_info.writers`` key, to
allow writing of custom egg-info files. The key ``egg_info.writers`` maps to a list of
strings, written as ``<FILENAME> = <module.function>``. Evidently these strings define the
new egg-info files and bound each file to a function. It is the job of the function to
write the file. The functions are defined in the present module.
'''

def write_git_sha_key(cmd, basename, filename):
    '''Store the git sha key to an egg-info file.

    Write the repository's git sha key to an egg-info file when the distribution attribute
    ``include_git_sha_key`` is ``True``. The attribute value is typically set by argument
    ``include_git_sha_key`` in ``setuptools.setup()``. Intended for use by setuptools only.

    Args:
       cmd (setuptools.command.egg_info.egg_info): Object creating a distribution's
           egg-info directory and contents.
       basename (str): Filename to write.
       filename (str): Complete path, with filename, to write.
    '''
    import os
    if cmd.distribution.include_git_sha_key:
        try:
            import git
            try:
                repo = git.Repo(".")
                git_sha_key = repo.head.object.hexsha
                if repo.is_dirty():
                    git_sha_key += " + uncommited changes (built from dirty repository)."
            except git.InvalidGitRepositoryError:
                git_sha_key = 'Could not obtain git sha key.'
        except ImportError:
            git_sha_key = 'Could not obtain git sha key.'
        argname = os.path.splitext(basename)[0]
        cmd.write_or_delete_file(argname, filename, git_sha_key)

def write_full_license(cmd, basename, filename):
    '''Store the licence statement to an egg-info file.

    Copy the full licence statement from a local file into an egg-info file, when the
    distribution attribute ``include_full_license`` is ``True``. The attribute value
    is typically set by argument ``include_full_license`` in ``setuptools.setup()``.
    Intended for use by setuptools only.

    Args:
       cmd (setuptools.command.egg_info.egg_info): Object creating a distribution's
           egg-info directory and contents.
       basename (str): Filename to write.
       filename (str): Complete path, with filename, to write.
    '''
    import os
    if cmd.distribution.include_full_license:
        argname = os.path.splitext(basename)[0]
        full_license_statement = open(basename).read()
        cmd.write_or_delete_file(argname, filename, full_license_statement)

def write_author_ids(cmd, basename, filename):
    '''Write the author IDs to an egg-info file.

    Copy the author IDs from a local file into an egg-info file, when the distribution
    attribute ``include_author_ids`` is ``True``. The attribute value is typically
    set by argument ``include_author_ids`` in ``setuptools.setup()``. Intended for use
    by setuptools only.

    Args:
       cmd (setuptools.command.egg_info.egg_info): Object creating a distribution's
           egg-info directory and contents.
       basename (str): Filename to write.
       filename (str): Complete path, with filename, to write.
    '''
    import os
    if cmd.distribution.include_author_ids:
        argname = os.path.splitext(basename)[0]
        author_ids = open(basename).read()
        cmd.write_or_delete_file(argname, filename, author_ids)

def write_qgis_path(cmd, basename, filename):
    '''Write the qgis path to an egg-info file.

    Write the qgis installation path to an egg-info file when the distribution attribute
    ``qgis_path`` is ``True``. The attribute value is typically set by argument ``qgis_path``
    in ``setup.py``. Intended for use by setuptools only.

    Args:
       cmd (setuptools.command.egg_info.egg_info): Object creating a distribution's
           egg-info directory and contents.
       basename (str): Filename to write.
       filename (str): Complete path, with filename, to write.
    '''
    import os
    if cmd.distribution.qgis_path:
        argname = os.path.splitext(basename)[0]
        cmd.write_or_delete_file(argname, filename, cmd.distribution.qgis_path)

def write_gmsh_bin_path(cmd, basename, filename):
    '''Write the gmsh binary path to an egg-info file.

    Write the gmsh binary installation path to an egg-info file when the distribution
    attribute ``gmsh_bin_path`` is ``True``. The attribute value is typically set by
    argument ``gmsh_bin_path`` in ``setup.py``. Intended for use by setuptools only.

    Args:
       cmd (setuptools.command.egg_info.egg_info): Object creating a distribution's
           egg-info directory and contents.
       basename (str): Filename to write.
       filename (str): Complete path, with filename, to write.
    '''
    import os
    if cmd.distribution.gmsh_bin_path:
        argname = os.path.splitext(basename)[0]
        cmd.write_or_delete_file(argname, filename, cmd.distribution.gmsh_bin_path)



#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.
''' setuptools-qmesh is a setuptools extension. As such it follows the
setuptools architecture:
'''

import pkg_resources
#Set the version attribute
try:
    __packaged_distro__ = pkg_resources.get_distribution('setuptools_qmesh')
    __version__ = __packaged_distro__.version
except (pkg_resources.DistributionNotFound, AttributeError):
    __version__ = None

del __packaged_distro__



#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.
''' Module with setuptools.Command-derived classes, checking for a functional qgis installation.
'''

from setuptools import Command

class CheckQgis(Command):
    '''Check qgis is installed.

    qmesh requires a functional installation of Qgis and the Qgis Python API to be present on the
    host system. Qgis and its Python API are not available through the Python Package Index
    (PyPI), so the user has to install them separately, perhaps through another package manager.
    The purpose of this class is to ensure Qgis and its Python API have been installed on the host
    system, and to verify their installation path, during installation of qmesh packages. The qgis
    installation path can be specified by the user as an option. If the path is not specified,
    methods in this class will attempt to locate it. Note that the qgis installation path is
    subsequently stored in an egg-info file (see the documentation of
    ``setuptools_qmesh.command.egg-info.write_qgis_path``). This class is derived from the
    ``setuptools`` Command class, to implement the necessary functionality in a way consistent to
    the ``setuptools`` packaging and distribution design, and to make the necessary checks at
    particular stages during the installation of qmesh packages.

    Technically it is possible to allow a particular command to be run separately, with a
    terminal command, after including a declaration of an entry-point in the call to
    ``setuptools.setup()``. The call arguments include a dictionary ``entry_points`` where a key
    ``distutils.commands`` is mapped to the custom commands. Such commands are specified as a
    list, where each list item bounds a custom command to a class: <command> = <module>:<class>
    The <class> is a derivative of the ``setuptools.Command`` class and the command can be run
    as ``python setup.py <command> <command-options>``. However, pip is a package management tool
    and it runs a very specific set of commands. While it is possible to run custom commands at the
    terminal with pip, it is elaborate to do so. Therefore the present command is intended to be
    run by other methods during installation. Nonetheless, the appropriate declarations in
    ``distutils.commands`` are made, for completeness. A command option named ``qgis-path`` is
    also introduced, via the ``user_options`` attribute. pip does not seem to facilitate passing
    command-line options to commands when the command is not explicitly run by the user. The user
    option is here included only for completeness and debugging. Otherwise, it is circumvented by
    the ``setuptools-qmesh`` package as follows: The value given with this command-line option
    (``--qgis-path=/path/to/qgis``) is stored in a ``setuptools.Command`` attribute named
    ``qgis_path``, not be confused with the similarly named Distribution object attribute.
    Projects that use setuptools-qmesh are built to look for the command-line option
    ``--qgis-path``, remove it from the options list and and pass its value to the ``setup()``
    call, setting the value of the Distribution object attribute, also named ``qgis_path``.
    Methods in this class will then copy the value of the Distribution attribute, or attempt to
    detect a valid qgis path if the option is absent.
    '''
    description = 'Check qgis is installed.'
    user_options = [('qgis-path=', None, "Path to qgis installation"),]
    qgis_path = None

    def initialize_options(self):
        '''Set command options to default values.

        In addition to the ``setuptools.Command`` attributes, the ``CheckQgis`` class also defines
        the attribute ``qgis_path``. According to ``setuptools`` standard practice, a method
        named ``initialize_options`` must set all attributes to an acceptable value. Therefore,
        the attribute ``qgis_path`` is given its default value in this method, ``None``.
        '''
        self.qgis_path = None

    def finalize_options(self):
        '''Set command options to final values, before running.

        The attribute ``qgis_path`` is set and checked by this method. The attribute
        ``qgis_path`` of this class should not be confused with the similarly named attribute
        of the distribution object. Packages that use ``setuptools-qmesh`` must scan the command
        line for the option ``--qgis-path`` and pass its value to the ``setup()`` call, thus
        assigning the distribution ``qgis_path`` attribute value. The present method will then
        assign the distribution attribute value to the Command (present class) attribute, using the
        following procedure: If the distribution object attribute is not ``None``, its value is
        assigned to the Command object attribute. Otherwise, the host system is probed for a qgis
        installation. If during probing a qgis installation is not found an exception is raised.
        '''
        from ..dist import assert_path
        from setuptools import distutils
        import subprocess
        #If the qgis_path attribute is not set, use the path specified by the distribution
        # attribute (via the setup.py file). If the distribution attribute is also None, try to
        # import qgis and figure out the path to the qgis installation tree.
        if self.qgis_path is None:
            if self.distribution.qgis_path is None:
                self.announce("No specification of qgis path is present."+
                              " Will try to detect a qgis installation", level=distutils.log.ERROR)
                try:
                    import qgis
                    qgis_python_path = qgis.__path__[0]
                    qgis_bin_path = subprocess.check_output(['which', 'qgis']).decode('utf-8')
                    qgis_path = '/'
                    for python_element, bin_element in zip(qgis_python_path.split('/'),
                                                           qgis_bin_path.split('/')):
                        if python_element == bin_element:
                            qgis_path = subprocess.os.path.join(qgis_path, python_element)
                        else:
                            break
                    self.qgis_path = qgis_path
                except ImportError:
                    msg = 'Host system does not appear to have a qgis installation.'
                    self.announce(msg, level=distutils.log.FATAL)
                    raise distutils.errors.DistutilsPlatformError(msg)
            else:
                self.qgis_path = self.distribution.qgis_path
        #Check that given qgis_path is a valid path
        assert_path(self.distribution, 'qgis-path', self.qgis_path)

    def run(self):
        '''Run check to ensure qgis is installed on host system.

        An attempt to find the qgis version is used as a check, to ensure a functional qgis
        installation. The location of the qgis installation tree is specified through the
        ``qgis_path`` attribute of this Command object and the Distribution object, see
        ``finalize_options`` doc-string.
        '''
        from setuptools import distutils
        self.distribution.announce('Looking for qgis installation at ' + self.qgis_path,
                                   level=distutils.log.INFO)
        try:
            import qgis.core
            qgs = qgis.core.QgsApplication([], False)
            qgs.setPrefixPath(self.qgis_path, True)
            qgs.initQgis()
            #From qgis2 to qgis3 the location of QGIS_VERSION changed
            try:
                qgis_version = qgis.core.QGis.QGIS_VERSION
            except AttributeError:
                qgis_version = qgis.core.Qgis.QGIS_VERSION
            self.distribution.announce('Found QGIS version ' + qgis_version + ' at ' +
                                       self.qgis_path, level=distutils.log.INFO)
        except ImportError:
            msg = 'Could not find qgis at ' + self.qgis_path
            self.announce(msg, level=distutils.log.FATAL)
            distutils.errors.DistutilsPlatformError(msg)
        #Assign the path to self.distribution.qgis_path, so that the egg writer
        # writes the path.
        self.distribution.qgis_path = self.qgis_path



#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.

'''Various functions bound to keyword arguments in setup.py

The map ``entry_points`` in ``setuptools.setup`` contains a ``distutils.setup_keywords``
key, to allow additions of custom keyword arguments. The key ``distutils.setup_keywords``
maps to a list of strings, written as ``<keyword> = <module.function>``. Evidently these
strings define the new keyword arguments and bound each argument to a function. The
functions are defined in the present module. Typically, the functions perform checks on the
keyword value, and then run instances of ``setuptools.Command``, as needed.
'''

def assert_path(dist, attr, value):
    '''Assert attribute describes a valid path.

       Various keyword variables, in calls to ``setuptools.setup``, allow the specification
       of paths where other qmesh packages should look for resources, such as third-party
       packages. This function asserts the value of such keywords is a string describing a
       valid path.

       Args:
           dist (setuptools.Distribution): The ``Distribution`` instance of ``setuptools``,
               orchestrating the installation or packaging procedure.
           attr (str): The name of the attribute, specified as a keyword argument in
               ``setuptools.setup`` calls.
           value (str): The value of the attribute (keyword), which should be a gmsh path.
       Raises:
           DistutilsSetupError: If the keyword value is not a valid path.
    '''
    from os import path
    from setuptools import distutils
    if not path.exists(value):
        msg = "%r must be a path (%r is not a path)" % (attr, value)
        dist.announce(msg, distutils.log.FATAL)
        raise distutils.errors.DistutilsSetupError(msg)



#!/usr/bin/env python

#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.
'''Module containing tests towards PEP8 compliance.

The pylint utility is used to check for PEP8 compliance. The file pylint.cfg
contains the configuration options for pylint, and it also points to spell checks
and spelling exceptions.
'''

import unittest

class TestPEP8(unittest.TestCase):
    '''Class for PEP8 compliance of setuptools-qmesh.

    Class (derived from unittest.TestCase) for PEP8 compliance testing.
    '''

    def setUp(self):
        '''Method setting-up test environment.

        This method is not aimed to be invoked by the user. It stores the path of
        the testing function, the project root path (setuptools-qmesh) and the
        path of the pylint configuration file.
        '''
        import os
        #Create logger object
        import logging
        self.log = logging.log
        #Construct absolute paths to test directory and project root directory
        self.this_path = os.path.dirname(os.path.realpath(__file__))
        self.project_test_path = os.path.split(self.this_path)[0]
        self.project_root_path = os.path.split(self.project_test_path)[0]
        #Store the path to the local pylint configuration file
        self.pylint_rcfile = os.path.join(self.this_path, 'pylint.cfg')

    def test_setup_script(self):
        '''Method testing the setup.py script, in setuptools-qmesh, for PEP8 compliance.
        '''
        import os
        #Construct path to the setup.py script
        setup_script_path = os.path.join(self.project_root_path, 'setup.py')
        #Invoke pylint on setup.py script
        self.invoke_pylint(setup_script_path)

    def test_setuptools_qmesh(self):
        '''Method testing setuptools-qmesh for PEP8 compliance.
        '''
        import os
        #Construct path to the setuptools_qmesh script
        setuptools_qmesh_path = os.path.join(self.project_root_path, 'setuptools_qmesh')
        #Invoke pylint on setuptools_qmesh package.
        self.invoke_pylint(setuptools_qmesh_path)

    def test_the_tests(self):
        '''Self-check: Method assessing the test code for PEP8 compliance.
        '''
        #Invoke pylint on this file.
        self.invoke_pylint(self.project_test_path)

    def invoke_pylint(self, file_or_path):
        '''Method facilitating pylint invocation.
        '''
        import tempfile
        import subprocess
        #Invoke pylint and capture pylint stdout & stderr to temporary files
        pylint_command = ['pylint', '--rcfile', self.pylint_rcfile, file_or_path]
        pylint_stdout = tempfile.TemporaryFile(mode='w+b')
        pylint_stderr = tempfile.TemporaryFile(mode='w+b')
        pylint_proc = subprocess.Popen(pylint_command, stdout=pylint_stdout, stderr=pylint_stderr)
        pylint_proc.wait()
        #If pylint had issues, echo its stdout and stdout
        if pylint_proc.returncode != 0:
            pylint_stdout.seek(0)
            self.log(40, '***pylint stdout:')
            for line in pylint_stdout:
                self.log(40, line.decode('utf-8').strip('\n'))
            pylint_stderr.seek(0)
            self.log(40, '***pylint stderr:')
            for line in pylint_stderr:
                self.log(40, line.decode('utf-8').strip('\n'))
        #Close files
        pylint_stdout.close()
        pylint_stderr.close()
        #Make sure pylint has run without issues
        self.assertEqual(pylint_proc.returncode, 0,
                         'pylint returned a non-zero code: '+str(pylint_proc.returncode)+'.'+\
                         ' Please check pylint output.')

if __name__ == '__main__':
    unittest.main()



#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.
''' Module with setuptools.Command-derived classes, checking for a functional gmsh installation.
'''

from setuptools import Command

class CheckGmsh(Command):
    '''Check gmsh is installed, by trying to find out installed version.

    qmesh requires a functional installation of the gmsh mesh generator to be present on the host
    system. Gmsh is not available through the Python Package Index (PyPI), so the user is required
    to install it separately, perhaps through another package manager. The purpose of this class
    is to ensure gmsh has been installed on the host system and verify the location of the gmsh
    binary (executable). The location of the gmsh binary can be specified by the user as an option.
    Thus, multiple versions of gmsh can be present in the host system, and the user can choose
    which one to use with a qmesh installation. If the location of the gmsh binary is not
    specified, methods in this class will attempt to locate it. Note that the location of the gmsh
    binary is subsequently stored in an egg-info file (see the documentation of
    ``setuptools_qmesh.command.egg-info.write_gmsh_bin_path``). This class is derived from the
    ``setuptools`` Command class, to implement the necessary functionality in a way consistent to
    the ``setuptools`` packaging and distribution design, and to make the necessary checks at
    particular stages during the installation of qmesh packages.

    Technically it is possible to allow a particular command to be run separately, with a
    terminal command, after including a declaration of an entry-point in the call to
    ``setuptools.setup()``. The call arguments include a dictionary ``entry_points`` where a key
    ``distutils.commands`` is mapped to the custom commands. Such commands are specified as a
    list, where each list item bounds a custom command to a class: <command> = <module>:<class>
    The <class> is a derivative of the ``setuptools.Command`` class and the command can be run
    as ``python setup.py <command> <command-options>``. However, pip is a package management tool
    and it runs a very specific set of commands. While it is possible to run custom commands at the
    terminal with pip, it is elaborate to do so. Therefore the present command is intended to be
    run by other methods during installation. Nonetheless, the appropriate declarations in
    ``distutils.commands`` are made, for completeness. A command option named ``gmsh-bin-path`` is
    also introduced, via the ``user_options`` attribute. pip does not seem to facilitate passing
    command-line options to commands when the command is not explicitly run by the user. The user
    option is here included only for completeness and debugging. Otherwise, it is circumvented by
    the ``setuptools-qmesh`` package as follows: The value given with this command-line option
    (``--gmsh-bin-path=/path/to/gmsh``) is stored in a ``setuptools.Command`` attribute named
    ``gmsh_bin_path``, not be confused with the similarly named Distribution object attribute.
    Projects that use setuptools-qmesh are built to look for the command-line option
    ``--gmsh-bin-path``, remove it from the options list and and pass its value to the ``setup()``
    call, setting the value of the Distribution object attribute, also named ``gmsh_bin_path``.
    Methods in this class will then copy the value of the Distribution attribute, or attempt to
    detect a valid gmsh path if the option is absent.
    '''
    description = 'Check gmsh is installed, by trying to find out installed version.'
    user_options = [('gmsh-bin-path=', None, "Path to gmsh binary (executable)"),]
    gmsh_bin_path = None

    def initialize_options(self):
        '''Set command options to default values.

        In addition to the ``setuptools.Command`` attributes, the ``CheckGmsh`` class also defines
        the attribute ``gmsh_bin_path``. According to ``setuptools`` standard practice, a method
        named ``initialize_options`` must initialise all attributes. Therefore, the attribute
        ``gmsh_bin_path`` is initialised in this method, and assigned a ``None`` value.
        '''
        self.gmsh_bin_path = None

    def finalize_options(self):
        '''Set command options to final values, before running.

        The attribute ``gmsh_bin_path`` is set and checked by this method. The attribute
        ``gmsh_bin_path`` of this class should not be confused with the similarly named attribute
        of the distribution object. Packages that use ``setuptools-qmesh`` must scan the command
        line for the option ``--gmsh-bin-path`` and pass its value to the ``setup()`` call, thus
        assigning the distribution ``gmsh_bin_path`` attribute value. The present method will then
        assign the distribution attribute value to the Command (present class) attribute, using the
        following procedure: If the distribution object attribute is not ``None``, its value is
        assigned to the Command object attribute. Otherwise, the host system is probed for a gmsh
        installation. If during probing a gmsh installation is not found an exception is raised.
        The specification of the gmsh binary path is allowed to be the absolute path to the
        directory containing the gmsh binary, or the absolute path including the gmsh binary
        filename at the end. This method will ensure the path including the filename is stored in
        ``gmsh_bin_path`` (e.g. ``/path/to/gmsh``) and then check the path exists.
        '''
        from ..dist import assert_path
        from setuptools import distutils
        import subprocess
        import platform
        #If the command-line option is not present, try to locate gmsh.
        if self.gmsh_bin_path is None:
            if self.distribution.gmsh_bin_path is None:
                self.announce("No specification of gmsh path is present."+
                              " Will try to detect a gmsh installation", level=distutils.log.ERROR)
                #Raise an exception if gmsh is not found.
                try:
                    if platform.system() == 'Linux' or platform.system() == 'Darwin':
                        gmsh_bin_path = subprocess.check_output(['which', 'gmsh']).decode('utf-8')
                    if platform.system() == 'Windows':
                        gmsh_bin_path = \
                                    subprocess.check_output(['where.exe', 'gmsh']).decode('utf-8')
                except subprocess.CalledProcessError:
                    msg = 'Could not locate a gmsh installation.'
                    self.announce(msg, level=distutils.log.FATAL)
                    distutils.errors.DistutilsSetupError(msg)
                self.gmsh_bin_path = gmsh_bin_path.strip()
            else:
                self.gmsh_bin_path = self.distribution.gmsh_bin_path
        #Check that given gmsh path is a valid path
        assert_path(self.distribution, 'gmsh_bin_path', self.gmsh_bin_path)

    def run(self):
        '''Run check to ensure gmsh is installed on host system.

        An attempt to find the gmsh version is used as a check, to ensure a functional gmsh
        installation. The location of the gmsh binary is specified through the ``gmsh_bin_path``
        attribute of this Command object and the Distribution object, see the documentation of the
        ``finalize_options`` method.
        '''
        from setuptools import distutils
        import subprocess
        import tempfile
        #If the path does not include the filename (gmsh) add it, and check if it exists
        if subprocess.os.path.isdir(self.gmsh_bin_path):
            self.gmsh_bin_path = subprocess.os.path.join(self.gmsh_bin_path, 'gmsh')
            if not subprocess.os.path.isfile(self.gmsh_bin_path):
                msg = 'Could not find gmsh at ' + self.gmsh_bin_path
                self.announce(msg, distutils.log.FATAL)
                raise distutils.errors.DistutilsSetupError(msg)
        #Call `/path/to/gmsh --version` to verify a functional gmsh installation.
        gmsh_stdout = tempfile.TemporaryFile(mode='w+b')
        gmsh_found = subprocess.Popen([self.gmsh_bin_path, '--version'],
                                      stdout=gmsh_stdout, stderr=gmsh_stdout)
        gmsh_found.wait()
        if gmsh_found.returncode != 0:
            msg = 'Could not find gmsh at ' + self.gmsh_bin_path
            self.announce(msg, distutils.log.FATAL)
            raise distutils.errors.DistutilsSetupError(msg)
        else:
            gmsh_stdout.seek(0)
            gmsh_version = gmsh_stdout.readline().decode("utf-8")
            self.announce("Found gmsh version " + gmsh_version.strip() +
                          " at " + self.gmsh_bin_path,
                          level=distutils.log.INFO)
        gmsh_stdout.close()
        #Assign the path to self.distribution.gmsh_bin_path, so that the egg writer
        # writes the path.
        self.distribution.gmsh_bin_path = self.gmsh_bin_path



#!/usr/bin/env python
#
#    Copyright (C) 2013 Alexandros Avdis and others.
#    See the AUTHORS.md file for a full list of copyright holders.
#
#    This file is part of setuptools-qmesh.
#
#    setuptools-qmesh is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    setuptools-qmesh is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with setuptools-qmesh.  If not, see <http://www.gnu.org/licenses/>.
'''Set-up script for installation and packaging of setuptools-qmesh.

This script facilitates installation and packaging of setuptools-qmesh, through standardised
procedures and utilities. This project need not be explicitly installed by a user:
setuptools-qmesh facilitates installation and packaging of other qmesh packages. In
particular, setuptools-qmesh introduces commands, setup-keywords and egg-info writers that
facilitate the correct operation, installation and packaging of other qmesh packages.
When ``pip`` is used, other qmesh packages will invoke installation of setuptools-qmesh.
In particular, a ``setuptools.setup`` call facilitates installation via ``pip``, or
``(sudo) python setup.py <command>``.
'''

def read(filename):
    '''Function reading file content.

    Function storing, and returning, file content as a string. Intended for reading the file
    containing the Version number and the read-me file during installation and packaging.

    Args:
        filename (str): Name of file to open and read contents.

    Returns:
        The file contents as a string (str).
    '''
    import os
    return open(os.path.join(os.path.dirname(__file__), filename)).read().strip()

def main():
    '''Function installing, packaging and testing the setuptools-qmesh package.

    This function makes a call to setuptools.setup, with appropriate arguments.
    '''
    import setuptools
    #Call setuptools setup() for installation, testing and packaging
    setuptools.setup(
        name='setuptools-qmesh',
        version=read('VERSION'),
        description="setuptools plugin for qmesh installation, packaging and testing",
        long_description=read('README.rst'),
        author="The QMesh Development Team.",
        author_email="develop@qmesh.org",
        url="https://www.qmesh.org",
        download_url='https://bitbucket.org/qmesh-developers/setuptools-qmesh/commits/tag/v'+\
                     read('VERSION'),
        packages=setuptools.find_packages(),
        entry_points={
            "distutils.commands":[
                "check_qgis = setuptools_qmesh.command.check_qgis:CheckQgis",
                "check_gmsh = setuptools_qmesh.command.check_gmsh:CheckGmsh"],
            "distutils.setup_keywords": [
                "qgis_path            = setuptools_qmesh.dist:assert_path",
                "gmsh_bin_path        = setuptools_qmesh.dist:assert_path",
                "include_git_sha_key  = setuptools.dist:assert_bool",
                "include_full_license = setuptools.dist:assert_bool",
                "include_author_ids   = setuptools.dist:assert_bool"],
            "egg_info.writers": [
                "QGIS_PATH     = setuptools_qmesh.command.egg_info:write_qgis_path",
                "GMSH_BIN_PATH = setuptools_qmesh.command.egg_info:write_gmsh_bin_path",
                "GIT_SHA_KEY   = setuptools_qmesh.command.egg_info:write_git_sha_key",
                "LICENSE       = setuptools_qmesh.command.egg_info:write_full_license",
                "AUTHORS.md    = setuptools_qmesh.command.egg_info:write_author_ids"]},
        license='GPLv3',
        install_requires=['GitPython'],
        test_suite="tests",
        keywords=['GIS', 'mesh generation', 'setuptools'],
        tests_require=['pylint', 'pyenchant',
                       'sphinxcontrib-bibtex', 'sphinxcontrib-napoleon'],
        extras_require=dict(build_doc=['sphinxcontrib-bibtex', 'sphinxcontrib-napoleon'])
        )

if __name__ == '__main__':
    main()
